#!/usr/bin/env python3

## GRAPHES TP n°12

#%%## Bibliothèques, modules

from math import inf,sqrt
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.text import Annotation
import matplotlib.lines as lines


#%%## Implémentation

#%%# Graphe G1 - Graphe associé à l'exemple du cours de parcours en largeur

G1 = {
      'A': {'adj': ['B', 'G'], 'XY': (1, 5), 'coul': 'blue'},
      'B': {'adj': ['A', 'C', 'D'], 'XY': (3, 5), 'coul': 'blue'},
      'C': {'adj': ['B'], 'XY': (5, 5), 'coul': 'yellow'},
      'D': {'adj': ['B', 'F', 'G'], 'XY': (3, 3), 'coul': 'yellow'},
      'E': {'adj': ['H'], 'XY': (5, 3), 'coul': 'yellow'},
      'F': {'adj': ['D'], 'XY': (1, 1), 'coul': 'yellow'},
      'G': {'adj': ['A', 'D'], 'XY': (3, 1), 'coul': 'blue'},
      'H': {'adj': ['E'], 'XY': (5, 1), 'coul': 'yellow'}
}

#%%# Graphe G2 - Graphe test
  
G2 = {
      'A': {'adj': ['B', 'G'], 'XY': (0, 2), 'coul': 'red'}, 
      'B': {'adj': ['A', 'C', 'D'], 'XY': (0, 3), 'coul': 'red'}, 
      'C': {'adj': ['B', 'D', 'J'], 'XY': (1, 2), 'coul': 'blue'}, 
      'D': {'adj': ['B', 'C', 'E'], 'XY': (2, 3), 'coul': 'blue'}, 
      'E': {'adj': ['D', 'F'], 'XY': (2, 4), 'coul': 'yellow'}, 
      'F': {'adj': ['E', 'L'], 'XY': (4, 4), 'coul': 'yellow'}, 
      'G': {'adj': ['A', 'H', 'I'], 'XY': (0, 1), 'coul': 'blue'}, 
      'H': {'adj': ['G', 'I'], 'XY': (1, 0), 'coul': 'yellow'}, 
      'I': {'adj': ['G', 'H', 'J', 'K'], 'XY': (2, 0), 'coul': 'yellow'}, 
      'J': {'adj': ['C', 'I', 'K', 'L'], 'XY': (3, 2), 'coul': 'yellow'}, 
      'K': {'adj': ['I', 'J', 'L'], 'XY': (4, 1), 'coul': 'yellow'}, 
      'L': {'adj': ['F', 'J', 'K'], 'XY': (4, 3), 'coul': 'yellow'}
}

#%%## Visualisation

BLANC  =  "yellow"      #  couleur  des  sommets  non  explorés
BLEU  =  "blue"        #  couleur  des  sommets  explorés (mais pas tous ses voisins)
ROUGE  =  "red"

def ext_comp(G,cle):
    gext = {}
    for i in G:
        gext[i]=G[i][cle]
    return gext

def convert_tot(dXY):
    x,y = [],[]
    for v in dXY.values():
        x.append(v[0])
        y.append(v[1])
    return x,y

def convert_tot_coul(dcouleurs):
    L = []
    for v in dcouleurs.values():
        L.append(v)
    return L

def trace_graphe_adj(G,transpg=0.5,transpa=0.15):
    dXY,dadj = ext_comp(G,'XY'), ext_comp(G,'adj')
    fig, ax = plt.subplots(figsize = (10,7))
    color = 'blue'
    area = (20)**2  # 10 point radii
    x,y = convert_tot(dXY)
    plt.scatter(x, y, s=area, c=color, alpha=transpg)
    for c in dadj :
        ax.annotate(c, dXY[c], size = 15, va="center", ha="center")
    for c in dadj:
        for j in dadj[c]:
            X = [dXY[c][0],dXY[j][0]] 
            Y = [dXY[c][1],dXY[j][1]]
            ax.plot(X, Y, color="blue", alpha = transpa)
    plt.show()
    
def trace_graphe_adj_cc(G, transpg=0.5):
    dXY,dadj,dcouleurs = ext_comp(G,'XY'), ext_comp(G,'adj'), ext_comp(G,'coul')
    transpa = 0.2
    fig, ax = plt.subplots(figsize = (10,7))
    area = (20)**2  # 10 point radii
    x,y = convert_tot(dXY)
    couleurs = convert_tot_coul(dcouleurs)
    plt.scatter(x, y, s=area, c=couleurs, alpha=transpg)
    for c in dadj :
        ax.annotate(c, dXY[c], size = 15, va="center", ha="center")
    for c in dadj:
        for j in dadj[c]:
            X = [dXY[c][0],dXY[j][0]] 
            Y = [dXY[c][1],dXY[j][1]]
            if dcouleurs[c] == BLANC or dcouleurs[j] == BLANC :
                coul,transpa = BLANC,1
            elif dcouleurs[c] == BLEU or dcouleurs[j] == BLEU :
                coul,transpa = BLEU,0.2
            else :
                coul,transpa = ROUGE,0.4
            ax.plot(X, Y, color=coul, alpha = transpa)
    plt.show() 

## 
from collections import deque

def init_parcours(G):
    # copie « profonde » du dictionnaire
    P = {s:G[s].copy() for s in G}
    # initialisation des sommets comme non marqués
    for s in P:
        P[s]["vu"]=False
        P[s]["coul"]=BLANC
    return(P)

def exploration_composante(G,r):
    G = init_parcours(G)
    F = deque([])
    E = []
    
    def enfile(s):
        G[s]["vu"]=True
        E.append(s)
        F.append(s)
        G[s]["coul"]=BLEU
    
    enfile(r)
    trace_graphe_adj_cc(G)
    while len(F)!=0:
        v = F.popleft()
        G[v]["coul"]=ROUGE
        for s in G[v]["adj"]:
            if not G[s]["vu"]:
                enfile(s)
                trace_graphe_adj_cc(G)
    return(E)
         